package kr.gmtc.gw.eyegw.db1;

import org.springframework.stereotype.Service;

@Service
public class EaiProcService {
    private EaiProcMapper ueaiProcMapper;
    
    public EaiProcService(EaiProcMapper ueaiProcMapper){
        this.ueaiProcMapper = ueaiProcMapper;
    }

	public void callProcArtsArr() {
    	
    	ueaiProcMapper.callProcArtsArr();
    	
    }
    
    public void callProcArtsDep() {
    	
    	ueaiProcMapper.callProcArtsDep();
    	
    }

    public void callProcAcdmArr() {
	
		ueaiProcMapper.callProcAcdmArr();
	
	}
	
	public void callProcAcdmDep() {
		
		ueaiProcMapper.callProcAcdmDep();
		
	}
	
	public void callProcGam() {
		
		ueaiProcMapper.callProcGam();
		
	}

	public void callProcAmos() {
		
		ueaiProcMapper.callProcAmos();
		
	}
	
	public void callProcNotam() {
		
		ueaiProcMapper.callProcNotam();
		
	}
    
}
